@@ -5,8 +5,9 @@
 #include <sys/stat.h>
 #include <fcntl.h>
 #include <unistd.h>
-#include <sys/ioctl.h>
 #include <sys/select.h>
+#include <asm/unistd.h>
+#include <errno.h>
 
 #include "inotify.h"
 
@@ -15,6 +16,14 @@
 #define ALL_MASK 0xffffffff
 #define EVENTQ_SIZE 128
 
+#define __NR_inotify_init	291
+#define __NR_inotify_add_watch	292
+#define __NR_inotify_rm_watch	293
+
+_syscall0(int, inotify_init);
+_syscall3(int, inotify_add_watch, int, fd, const char *, name, __u32, mask);
+_syscall2(int, inotify_rm_watch, int, fd, __u32, wd);
+
 struct inotify_event *eventq[EVENTQ_SIZE];
 int eventq_head = 0;
 int eventq_tail = 0;
@@ -23,11 +32,12 @@ int open_dev ()
 {
 	int fd;
 
-	fd = open("/dev/inotify", O_RDONLY);
+	fd = inotify_init ();
 
-	if (fd < 0) {
-		perror ("open(\"/dev/inotify\", O_RDONLY) = ");
-	}
+	if (fd < 0)
+		perror ("inotify_init");
+
+	printf("inotify device fd = %d\n", fd);
 
 	return fd;
 }
@@ -113,8 +123,10 @@ int read_events (queue_t q, int fd)
 
 	r = read (fd, buffer, 16384);
 
-	if (r <= 0)
+	if (r <= 0) {
+		perror ("read(fd, buffer, 16384) = ");
 		return r;
+	}
 
 	buffer_i = 0;
 	printf("read = %d\n", r);
@@ -174,25 +186,11 @@ int read_and_print_events (queue_t q, int fd)
 
 int watch_dir (int fd, const char *dirname, __u32 mask)
 {
-	struct inotify_watch_request iwr;
 	int wd;
-	int file;
-
-	file = open (dirname, O_RDONLY);
-	if (file < 0) {
-		perror ("open");
-		return -1;
-	}
-
-	iwr.fd = file;
-	iwr.mask = mask;
-
-	wd = ioctl(fd, INOTIFY_WATCH, &iwr);
 
+	wd = inotify_add_watch (fd, dirname, mask);
 	if (wd < 0)
-		perror ("ioctl");		
-
-	close (file);
+		perror ("inotify_add_watch");
 
 	printf("%s WD=%d\n", dirname, wd);
 	return wd;
@@ -200,13 +198,12 @@ int watch_dir (int fd, const char *dirname, __u32 mask)
 
 int ignore_wd (int fd, int wd)
 {
-	int r;
+	int ret;
 
-	r = ioctl(fd, INOTIFY_IGNORE, &wd);
+	ret = inotify_rm_watch (fd, wd);
 
-	if (r < 0) {
-		perror("ioctl(fd, INOTIFY_IGNORE, &wid) = ");
-	}
+	if (ret < 0)
+		perror("inotify_rm_watch");
 
-	return r;
+	return ret;
 }
